@@ -17,8 +17,10 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+#include <libgnome/libgnome.h>
 #include <signal.h>
+#include <unistd.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
@@ -46,7 +48,6 @@
 #include "verify.h"
 #include "display.h"
 #include "choose.h"
-#include "errorgui.h"
 #include "gdm-net.h"
 
 /* Local functions */
@@ -73,6 +74,8 @@ gid_t GdmGroupId;		/* Groupid under which gdm should run */
 pid_t extra_process = -1;	/* An extra process.  Used for quickie 
 				   processes, so that they also get whacked */
 int extra_status = 0;		/* Last status from the last extra process */
+gboolean preserve_ld_vars = FALSE; /* Preserve the ld environment variables */
+gboolean no_daemon = FALSE;	/* Do not daemonize */
 
 GdmConnection *fifoconn = NULL; /* Fifo connection */
 GdmConnection *unixconn = NULL; /* UNIX Socket connection */
@@ -156,6 +159,8 @@ char *stored_path = NULL;
 
 static gboolean gdm_restart_mode = FALSE;
 
+static GMainLoop *main_loop = NULL;
+
 static gboolean
 display_exists (int num)
 {
@@ -461,7 +466,7 @@ gdm_config_parse (void)
 		    char *s = g_strdup_printf (_("Xdmcp is disabled and gdm "
 						 "cannot find any local server "
 						 "to start.  Aborting!  Please "
-						 "correct the configuration %s "
+						 "correct the configuration %s"
 						 "and restart gdm."),
 					       GDM_CONFIG_FILE);
 		    gdm_text_message_dialog (s);
@@ -597,7 +602,7 @@ gdm_config_parse (void)
 	    char *s = g_strdup_printf (_("Server Authorization directory "
 					 "(daemon/ServAuthDir) is set to %s "
 					 "but has the wrong permissions, it "
-					 "should have permissions of 0750. "
+					 "should have permissions of 0750."
 					 "Please correct the permissions or "
 					 "the gdm configuration %s and "
 					 "restart gdm."),
@@ -809,17 +814,17 @@ deal_with_x_crashes (GdmDisplay *d)
 
 		    /* unset DISPLAY and XAUTHORITY if they exist
 		     * so that gdialog (if used) doesn't get confused */
-		    ve_unsetenv ("DISPLAY");
-		    ve_unsetenv ("XAUTHORITY");
+		    gnome_unsetenv ("DISPLAY");
+		    gnome_unsetenv ("XAUTHORITY");
 
 		    /* some promised variables */
-		    ve_setenv ("XLOG", xlog, TRUE);
-		    ve_setenv ("BINDIR", EXPANDED_BINDIR, TRUE);
-		    ve_setenv ("SBINDIR", EXPANDED_SBINDIR, TRUE);
+		    gnome_setenv ("XLOG", xlog, TRUE);
+		    gnome_setenv ("BINDIR", EXPANDED_BINDIR, TRUE);
+		    gnome_setenv ("SBINDIR", EXPANDED_SBINDIR, TRUE);
 
 		    /* To enable gettext stuff in the script */
-		    ve_setenv ("TEXTDOMAIN", PACKAGE, TRUE);
-		    ve_setenv ("TEXTDOMAINDIR", GNOMELOCALEDIR, TRUE);
+		    gnome_setenv ("TEXTDOMAIN", PACKAGE, TRUE);
+		    gnome_setenv ("TEXTDOMAINDIR", GNOMELOCALEDIR, TRUE);
 
 		    execv (argv[0], argv);
 	
@@ -1145,7 +1150,7 @@ gdm_cleanup_children (void)
     if (gdm_restart_mode)
 	    gdm_safe_restart ();
 
-    gdm_quit ();
+    g_main_loop_quit (main_loop);
 
     return TRUE;
 }
@@ -1156,7 +1161,7 @@ gdm_restart_now (void)
 	gdm_info (_("Gdm restarting ..."));
 	final_cleanup ();
 	if (stored_path != NULL)
-		ve_setenv ("PATH", stored_path, TRUE);
+		gnome_setenv ("PATH", stored_path, TRUE);
 	execvp (stored_argv[0], stored_argv);
 	gdm_error (_("Failed to restart self"));
 	_exit (1);
@@ -1214,15 +1219,13 @@ mainloop_sig_callback (gint8 sig, gpointer data)
 static void
 signal_notify (int sig)
 {
-	g_signal_notify (sig);
+	gdm_signal_notify (sig);
 }
 
 /*
  * main: The main daemon control
  */
 
-static GMainLoop *main_loop;    
-
 static void
 store_argv (int argc, char *argv[])
 {
@@ -1261,6 +1264,15 @@ create_connections (void)
 					    NULL /* destroy_notify */);
 }
 
+struct poptOption options [] = {
+	{ "nodaemon", '\0', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH,
+	  &no_daemon, 0, N_("Do not fork into the background"), NULL },
+	{ "preserve-ld-vars", '\0', POPT_ARG_NONE,
+	  &preserve_ld_vars, 0, N_("Preserve LD_* variables"), NULL },
+        POPT_AUTOHELP
+	{ NULL, 0, 0, NULL, 0}
+};
+
 int 
 main (int argc, char *argv[])
 {
@@ -1274,66 +1286,24 @@ main (int argc, char *argv[])
     bindtextdomain(PACKAGE, GNOMELOCALEDIR);
     textdomain(PACKAGE);
 
-    /* This is an utter hack.  BUT we don't want to have the message in
-     * another program as it is supposed to be a last ditch effort to talk
-     * to the user, so what happens is that we respawn ourselves with
-     * an argument --run-error-dialog error dialog_type <x>:<y>:<width>:<height>
-     * the coordinates are the coordinates of this screen (when using
-     * xinerama) they can all be 0 */
-    if (argc == 5 &&
-	strcmp (argv[1], "--run-error-dialog") == 0) {
-	    int x = 0, y = 0, width = 0, height = 0;
-	    sscanf (argv[4], "%d:%d:%d:%d", &x, &y, &width, &height);
-	    gdm_run_errorgui (argv[2], argv[3], x, y, width, height);
-	    _exit (0);
-    }
-    /* This is another utter hack.  Same as above but for questions */
-    if (argc == 5 &&
-	strcmp (argv[1], "--run-failsafe-question") == 0) {
-	    int x = 0, y = 0, width = 0, height = 0;
-	    gboolean echo;
-	    char *ret;
-	    sscanf (argv[4], "%d:%d:%d:%d", &x, &y, &width, &height);
-	    if (strcmp (argv[3], "FALSE") == 0)
-		    echo = FALSE;
-	    else
-		    echo = TRUE;
-	    ret = gdm_run_failsafe_question (argv[2], echo, x, y, width, height);
-	    if (ret != NULL) {
-		    g_print (ret);
-		    fflush (stdout);
-	    }
-	    _exit (0);
-    }
-    /* This is another utter hack.  Same as above but for yes/no questions */
-    if (argc == 4 &&
-	strcmp (argv[1], "--run-failsafe-yesno") == 0) {
-	    int x = 0, y = 0, width = 0, height = 0;
-	    gboolean ret;
-	    sscanf (argv[3], "%d:%d:%d:%d", &x, &y, &width, &height);
-	    ret = gdm_run_failsafe_yesno (argv[2], x, y, width, height);
-	    if (ret)
-		    g_print ("yes\n");
-	    else
-		    g_print ("no\n");
-	    fflush (stdout);
-	    _exit (0);
-    }
-
-
     /* XDM compliant error message */
-    if (getuid() != 0) {
+    if (getuid () != 0) {
 	    /* make sure the pid file doesn't get wiped */
 	    GdmPidFile = NULL;
 	    gdm_fail (_("Only root wants to run gdm\n"));
     }
 
-
     /* Initialize runtime environment */
     umask (022);
-    gnome_do_not_create_directories = TRUE;
-    gnomelib_init ("gdm", VERSION);
-    main_loop = g_main_new (FALSE);
+
+    gnome_program_init ("gdm", VERSION,
+			LIBGNOME_MODULE /* module_info */,
+			argc, argv,
+			GNOME_PARAM_POPT_TABLE, options,
+			GNOME_PARAM_CREATE_DIRECTORIES, FALSE,
+			NULL);
+
+    main_loop = g_main_loop_new (NULL, FALSE);
     openlog ("gdm", LOG_PID, LOG_DAEMON);
 
 
@@ -1361,7 +1331,7 @@ main (int argc, char *argv[])
     }
 
     /* Become daemon unless started in -nodaemon mode or child of init */
-    if ( (argc == 2 && strcmp (argv[1], "-nodaemon") == 0) || getppid() == 1) {
+    if (no_daemon || getppid() == 1) {
 
 	/* Write pid to pidfile */
 	if ((pf = fopen (GdmPidFile, "w"))) {
@@ -1373,11 +1343,11 @@ main (int argc, char *argv[])
 	gdm_daemonify();
 
     /* Signal handling */
-    g_signal_add (SIGCHLD, mainloop_sig_callback, NULL);
-    g_signal_add (SIGTERM, mainloop_sig_callback, NULL);
-    g_signal_add (SIGINT, mainloop_sig_callback, NULL);
-    g_signal_add (SIGHUP, mainloop_sig_callback, NULL);
-    g_signal_add (SIGUSR1, mainloop_sig_callback, NULL);
+    gdm_signal_add (SIGCHLD, mainloop_sig_callback, NULL);
+    gdm_signal_add (SIGTERM, mainloop_sig_callback, NULL);
+    gdm_signal_add (SIGINT, mainloop_sig_callback, NULL);
+    gdm_signal_add (SIGHUP, mainloop_sig_callback, NULL);
+    gdm_signal_add (SIGUSR1, mainloop_sig_callback, NULL);
     
     term.sa_handler = signal_notify;
     term.sa_flags = SA_RESTART;
@@ -1438,7 +1408,7 @@ main (int argc, char *argv[])
      */
     while (1)
       {
-        gdm_run ();
+	g_main_loop_run (main_loop);
 	gdm_debug ("main: Exited main loop");
       }
 
@@ -1448,116 +1418,97 @@ main (int argc, char *argv[])
 /* signal main loop support */
 
 
-typedef struct _GSignalData GSignalData;
-struct _GSignalData
-{
-  guint8      index;
-  guint8      shift;
-  GSignalFunc callback;
+typedef struct _SignalSource SignalSource;
+struct _SignalSource {
+	GSource     source;
+
+	gint8       signal;
+	guint8      index;
+	guint8      shift;
 };
 
-static gboolean g_signal_prepare  (gpointer  source_data,
-				   GTimeVal *current_time,
-				   gint     *timeout,
-				   gpointer   user_data);
-static gboolean g_signal_check    (gpointer  source_data,
-				   GTimeVal *current_time,
-				   gpointer  user_data);
-static gboolean g_signal_dispatch (gpointer  source_data,
-				   GTimeVal *current_time,
-				   gpointer  user_data);
-
-static GSourceFuncs signal_funcs = {
-  g_signal_prepare,
-  g_signal_check,
-  g_signal_dispatch,
-  g_free
-};
 static	guint32	signals_notified[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
 static gboolean
-g_signal_prepare (gpointer  source_data,
-		  GTimeVal *current_time,
-		  gint     *timeout,
-		  gpointer  user_data)
+gdm_signal_prepare (GSource  *source,
+		    gint     *timeout)
 {
-  GSignalData *signal_data = source_data;
-  
-  return signals_notified[signal_data->index] & (1 << signal_data->shift);
+	SignalSource *ss = (SignalSource *)source;
+
+	return signals_notified[ss->index] & (1 << ss->shift);
 }
 
 static gboolean
-g_signal_check (gpointer  source_data,
-		GTimeVal *current_time,
-		gpointer  user_data)
+gdm_signal_check (GSource *source)
 {
-  GSignalData *signal_data = source_data;
-  
-  return signals_notified[signal_data->index] & (1 << signal_data->shift);
+	SignalSource *ss = (SignalSource *)source;
+
+	return signals_notified[ss->index] & (1 << ss->shift);
 }
 
 static gboolean
-g_signal_dispatch (gpointer  source_data,
-		   GTimeVal *current_time,
-		   gpointer  user_data)
+gdm_signal_dispatch (GSource     *source,
+		     GSourceFunc  callback,
+		     gpointer     user_data)
 {
-  GSignalData *signal_data = source_data;
-  
-  signals_notified[signal_data->index] &= ~(1 << signal_data->shift);
-  
-  return signal_data->callback (-128 + signal_data->index * 32 + signal_data->shift, user_data);
+	SignalSource *ss = (SignalSource *)source;
+
+	signals_notified[ss->index] &= ~(1 << ss->shift);
+
+	return ((GDMSignalFunc)callback) (ss->signal, user_data);
+}
+
+static GSourceFuncs signal_funcs = {
+	gdm_signal_prepare,
+	gdm_signal_check,
+	gdm_signal_dispatch
+};
+
+guint
+gdm_signal_add (gint8	      signal,
+		GDMSignalFunc function,
+		gpointer      data)
+{
+	return gdm_signal_add_full (G_PRIORITY_DEFAULT, signal, function, data, NULL);
 }
 
 guint
-g_signal_add (gint8	  signal,
-	      GSignalFunc function,
-	      gpointer    data)
+gdm_signal_add_full (gint           priority,
+		     gint8          signal,
+		     GDMSignalFunc  function,
+		     gpointer       data,
+		     GDestroyNotify destroy)
 {
-  return g_signal_add_full (G_PRIORITY_DEFAULT, signal, function, data, NULL);
-}
+	GSource *source;
+	SignalSource *ss;
+	guint s = 128 + signal;
 
-guint
-g_signal_add_full (gint           priority,
-		   gint8          signal,
-		   GSignalFunc    function,
-		   gpointer       data,
-		   GDestroyNotify destroy)
-{
-  GSignalData *signal_data;
-  guint s = 128 + signal;
-  
-  g_return_val_if_fail (function != NULL, 0);
-  
-  signal_data = g_new (GSignalData, 1);
-  signal_data->index = s / 32;
-  signal_data->shift = s % 32;
-  signal_data->callback = function;
-  
-  return g_source_add (priority, TRUE, &signal_funcs, signal_data, data, destroy);
+	g_return_val_if_fail (function != NULL, 0);
+
+	source = g_source_new (&signal_funcs, sizeof (SignalSource));
+	ss = (SignalSource *)source;
+
+	ss->signal = signal;
+	ss->index = s / 32;
+	ss->shift = s % 32;
+
+	g_source_set_priority (source, priority);
+	g_source_set_callback (source, (GSourceFunc)function, data, destroy);
+	g_source_set_can_recurse (source, TRUE);
+
+	return g_source_attach (source, NULL);
 }
 
 void
-g_signal_notify (gint8 signal)
+gdm_signal_notify (gint8 signal)
 {
-  guint index, shift;
-  guint s = 128 + signal;
-  
-  index = s / 32;
-  shift = s % 32;
-  
-  signals_notified[index] |= 1 << shift;
-}
+	guint index, shift;
+	guint s = 128 + signal;
 
-void
-gdm_run (void)
-{
-  g_main_run (main_loop);
-}
+	index = s / 32;
+	shift = s % 32;
 
-void
-gdm_quit (void)
-{
-  g_main_quit (main_loop);
+	signals_notified[index] |= 1 << shift;
 }
 
 static void
